@@ -19,6 +19,9 @@
 
 #include <string.h>
 #include <map>
+#include <cairo.h>
+#include <librsvg/rsvg.h>
+#include <librsvg/rsvg-cairo.h>
 
 #include "actor.h"
 #include "kmplayercontrol.h"
@@ -94,6 +97,7 @@ static bool isPlayListMime (const String & mime) {
             !strcmp (mimestr, "application/rss+xml") ||
             !strcmp (mimestr, "application/atom+xml") ||
             !strcmp (mimestr, "image/vnd.rn-realpix") ||
+            !strcmp (mimestr, "image/svg+xml") ||
             !strcmp (mimestr, "application/x-mplayer2"));
 }
 
@@ -162,7 +166,7 @@ KDE_NO_EXPORT bool MediaInfo::wget (const String &str) {
 
     if (mrl) {
         if (mime.isEmpty ()) {
-            for (NodePtr p = mrl->parentNode (); p; p = p->parentNode ()) {
+            for (Node *p = mrl->parentNode (); p; p = p->parentNode ()) {
                 Mrl *m = p->mrl ();
                 if (m && m->linkNode ()->audio_only) {
                     mrl->linkNode ()->audio_only = true;
@@ -216,11 +220,11 @@ KDE_NO_EXPORT bool MediaInfo::wget (const String &str) {
 KDE_NO_EXPORT bool MediaInfo::readChildDoc () {
     TextStream textstream (data);
     String line;
+    NodePtr cur_elm = node;
     do {
         line = textstream.readLine ();
     } while (!line.isNull () && line.stripWhiteSpace ().isEmpty ());
     if (!line.isNull ()) {
-        NodePtr cur_elm = node;
         if (cur_elm->isPlayable ())
             cur_elm = cur_elm->mrl ()->linkNode ();
         if (cur_elm->mrl ()->mimetype == String ("audio/x-scpls")) {
@@ -299,6 +303,7 @@ KDE_NO_EXPORT bool MediaInfo::readChildDoc () {
             }
         }
     }
+    return !cur_elm->isPlayable ();
 }
 
 KDE_NO_EXPORT String MediaInfo::mimetype () {
@@ -330,6 +335,14 @@ KDE_NO_EXPORT void MediaInfo::create () {
                     media = new AudioVideoActor (mgr, node);
                 break;
             case ActorAgent::Image:
+                if (data.size () && mime == "image/svg+xml") {
+                    readChildDoc ();
+                    if (node->firstChild () &&
+                            id_node_svg == node->lastChild ()->id) {
+                        media = new ImageActor (node);
+                        break;
+                    }
+                }
                 if (data.size () &&
                         (!(mimetype ().startsWith ("text/") ||
                            mime == "image/vnd.rn-realpix") ||
@@ -469,13 +482,30 @@ ImageData::~ImageData() {
 void ImageData::setImage (Image *img) {
     if (image != img) {
         delete image;
-        if (surface)
+        if (surface) {
             cairo_surface_destroy (surface);
+            surface = NULL;
+        }
         image = img;
-        width = img->width ();
-        height = img->height ();
-        has_alpha = false;
+        if (img) {
+            width = img->width ();
+            height = img->height ();
+            has_alpha = false;
+        }
     }
+    if (!img)
+        width = height = 0;
+}
+
+void ImageData::setImage (cairo_surface_t *s, int w, int h) {
+    delete image;
+    image = NULL;
+    if (surface)
+        cairo_surface_destroy (surface);
+    surface = s;
+    width = w;
+    height = h;
+    has_alpha = true;
 }
 
 ImageActor::ImageActor (ActorAgent *manager, Node *node,
@@ -483,9 +513,10 @@ ImageActor::ImageActor (ActorAgent *manager, Node *node,
  : Actor (manager, node),
    img_movie (NULL),
    img_movie_iter (NULL),
-   pix_loader (NULL),
+   svg_renderer (NULL),
    img_movie_timer (0),
-   timeout (0) {
+   timeout (0),
+   update_render (false) {
     setupImage (url, ba);
 }
 
@@ -494,19 +525,38 @@ ImageActor::ImageActor (Node *node, ImageDataPtr id)
          node),
    img_movie (NULL),
    img_movie_iter (NULL),
-   pix_loader (NULL),
+   svg_renderer (NULL),
    img_movie_timer (0),
-   timeout (0) {
-    cached_img = id;
+   timeout (0),
+   update_render (false) {
+    if (!id) {
+        Node *c = findChildWithId (node, id_node_svg);
+        if (c) {
+            svg_renderer = rsvg_handle_new ();
+            String xml = c->outerXML();
+            const char *cxml = (const char*) xml;
+            bool success = rsvg_handle_write (svg_renderer,
+                    (const guchar*)cxml, strlen (cxml), NULL);
+            rsvg_handle_close (svg_renderer, NULL);
+            if (success) {
+                cached_img = new ImageData (String ());
+                cached_img->flags = ImageData::ImageScalable;
+            } else {
+                g_object_unref (svg_renderer);
+                svg_renderer = NULL;
+            }
+        }
+    } else {
+        cached_img = id;
+    }
 }
 
 ImageActor::~ImageActor () {
     stop ();
     if (img_movie_iter)
         g_object_unref (img_movie_iter);
-    if (pix_loader)
-        //gdk_pixbuf_loader_close (pix_loader, 0L);
-        g_object_unref (pix_loader);
+    if (svg_renderer)
+        g_object_unref (svg_renderer);
     else if (img_movie)
         g_object_unref (img_movie);
 }
@@ -527,6 +577,72 @@ void ImageActor::pause () {
     img_movie_timer = 0;
 }
 
+KDE_NO_EXPORT
+unsigned ImageActor::render (cairo_surface_t *similar, int width, int height) {
+    unsigned penalty = 0;
+    if (svg_renderer && update_render) {
+        g_object_unref (svg_renderer);
+        svg_renderer = NULL;
+        Node *c = findChildWithId (m_node, id_node_svg);
+        if (c) {
+            RsvgHandle *r = rsvg_handle_new ();
+            String xml = c->outerXML();
+            const char *cxml = (const char*) xml;
+            size_t len = strlen (cxml);
+            bool success = rsvg_handle_write (r,
+                    (const guchar*)cxml, len, NULL);
+            rsvg_handle_close (r, NULL);
+            if (success) {
+                cached_img->setImage (NULL);
+                svg_renderer = r;
+            } else {
+                g_object_unref (r);
+            }
+            penalty += len / 1024;
+        }
+        update_render = false;
+    }
+    if (svg_renderer &&
+            (cached_img->width != width || cached_img->height != height)) {
+        RsvgDimensionData dim;
+        rsvg_handle_get_dimensions (svg_renderer, &dim);
+        cairo_surface_t *surf = cairo_surface_create_similar (similar,
+                CAIRO_CONTENT_COLOR_ALPHA, width, height);
+        cairo_t *cr = cairo_create (surf);
+        cairo_scale (cr, 1.0 * width / dim.width, 1.0 * height / dim.height);
+        rsvg_handle_render_cairo (svg_renderer, cr);
+        cairo_destroy (cr);
+        cached_img->setImage (surf, width, height);
+
+        penalty += width * height / (10 * 480);
+        //Image *img = new Image (GPixbuf (
+        //            rsvg_handle_get_pixbuf (svg_renderer), true));
+        //cached_img->setImage (img);
+    }
+    return penalty;
+}
+
+KDE_NO_EXPORT void ImageActor::updateRender () {
+    update_render = true;
+    if (m_node)
+        m_node->document()->post(m_node, new Posting (m_node, MsgMediaUpdated));
+}
+
+KDE_NO_EXPORT void ImageActor::sizes (Single &width, Single &height) {
+    if (svg_renderer) {
+        RsvgDimensionData dim;
+        rsvg_handle_get_dimensions (svg_renderer, &dim);
+        width = dim.width;
+        height = dim.height;
+    } else if (cached_img) {
+        width = cached_img->width;
+        height = cached_img->height;
+    } else {
+        width = 0;
+        height = 0;
+    }
+}
+
 gboolean movieUpdateTimeout (void * p) {
    return ((ImageActor *)p)->movieTimer ();
 }
@@ -566,13 +682,15 @@ KDE_NO_EXPORT
 void ImageActor::setupImage (const String &url, const ByteArray &data) {
     GError *gerror = 0L;
 
-    pix_loader = gdk_pixbuf_loader_new ();
+    GdkPixbufLoader *pix_loader = gdk_pixbuf_loader_new ();
     if (gdk_pixbuf_loader_write (pix_loader,
                 (const guchar *) data.data (),
                 data.size (), &gerror) &&
             gdk_pixbuf_loader_close (pix_loader, 0L)) {
         cached_img = ImageDataPtr (new ImageData (url));
         img_movie = gdk_pixbuf_loader_get_animation (pix_loader);
+        if (img_movie)
+            g_object_ref (img_movie);
     }
 
     if (img_movie && !gdk_pixbuf_animation_is_static_image (img_movie)) {
@@ -586,11 +704,12 @@ void ImageActor::setupImage (const String &url, const ByteArray &data) {
         cached_img->flags = (int) ImageData::ImagePixmap;
         (*image_data_map)[url] = ImageDataPtrW (cached_img);
     }
+    g_object_unref (pix_loader);
     g_free (gerror);
 }
 
 bool ImageActor::isEmpty () const {
-    return !cached_img || cached_img->isEmpty ();
+    return !cached_img || (!svg_renderer && cached_img->isEmpty ());
 }
 
 //------------------------%<----------------------------------------------------
