@@ -26,7 +26,7 @@
 
 typedef struct _GdkPixbufAnimation GdkPixbufAnimation;
 typedef struct _GdkPixbufAnimationIter GdkPixbufAnimationIter;
-typedef struct _GdkPixbufLoader GdkPixbufLoader;
+typedef struct _RsvgHandle RsvgHandle;
 
 namespace KMPlayer {
 
@@ -156,6 +156,7 @@ struct KMPLAYER_NO_EXPORT ImageData {
     ImageData( const String & img);
     ~ImageData();
     void setImage (Image *img);
+    void setImage (cairo_surface_t *s, int w, int h);
     void copyImage (Surface *s, int w, int h, cairo_surface_t *similar,
             unsigned int *penalty, CalculatedSizer *zoom=NULL);
     bool isEmpty () const {
@@ -178,7 +179,7 @@ class KMPLAYER_NO_EXPORT ImageActor : public Actor {
 public:
     ImageActor (ActorAgent *manager, Node *node,
             const String &url, const ByteArray &data);
-    ImageActor (Node *node, ImageDataPtr id);
+    ImageActor (Node *node, ImageDataPtr id = NULL);
 
     ActorAgent::ActorType type () const { return ActorAgent::Image; }
 
@@ -188,6 +189,9 @@ public:
     void unpause ();
 
     bool isEmpty () const;
+    unsigned render (cairo_surface_t *similar, int width, int height);
+    void sizes (Single &width, Single &height);
+    void updateRender ();
     bool movieTimer ();
 
     ImageDataPtr cached_img;
@@ -200,9 +204,10 @@ private:
 
     GdkPixbufAnimation *img_movie;
     GdkPixbufAnimationIter *img_movie_iter;
-    GdkPixbufLoader *pix_loader;
+    RsvgHandle *svg_renderer;
     int img_movie_timer;
     int timeout;
+    bool update_render;
 };
 
 //------------------------%<----------------------------------------------------
