@@ -33,6 +33,12 @@ const short id_node_link = 302;
 const short id_node_title = 303;
 const short id_node_summary = 304;
 const short id_node_content = 305;
+const short id_node_media_group = 306;
+const short id_node_media_content = 307;
+const short id_node_media_title = 308;
+const short id_node_media_description = 309;
+const short id_node_media_thumbnail = 310;
+const short id_node_ignored = 311;
 
 /**
  * '<feed>' tag
@@ -43,13 +49,14 @@ public:
     Node *childFromTag (const String & tag);
     KDE_NO_EXPORT const char * nodeName () const { return "feed"; }
     void closed ();
-    bool expose () const { return !caption.isEmpty (); }
+    bool expose () const { return !title.isEmpty (); }
 };
 
 class KMPLAYER_NO_EXPORT Entry : public Mrl {
 public:
     KDE_NO_CDTOR_EXPORT Entry (NodePtr & d) : Mrl (d, id_node_entry) {}
     Node *childFromTag (const String & tag);
+    String description () const;
     KDE_NO_EXPORT const char * nodeName () const { return "entry"; }
     PlayType playType () { return play_type_none; }
     void closed ();
@@ -60,6 +67,7 @@ public:
     KDE_NO_CDTOR_EXPORT Link (NodePtr & d) : Mrl (d, id_node_link) {}
     KDE_NO_EXPORT const char * nodeName () const { return "link"; }
     PlayType playType ();
+    bool expose () const { return !src.isEmpty (); }
     void closed ();
 };
 
@@ -69,7 +77,25 @@ public:
     KDE_NO_EXPORT const char * nodeName () const { return "content"; }
     PlayType playType ();
     void closed ();
-    //bool expose () const { return isPlayable (); }
+    bool expose () const { return !src.isEmpty (); }
+};
+
+class KMPLAYER_NO_EXPORT MediaGroup : public Title {
+public:
+    MediaGroup (NodePtr &d) : Title (d, id_node_media_group) {}
+    Node *childFromTag (const String &tag);
+    void *message (MessageType msg, void *content=NULL);
+    KDE_NO_EXPORT const char *nodeName () const { return "media:group"; }
+    void closed ();
+    bool expose () const { return false; }
+};
+
+class KMPLAYER_NO_EXPORT MediaContent : public Mrl {
+public:
+    MediaContent (NodePtr &d) : Mrl (d, id_node_media_content) {}
+    KDE_NO_EXPORT const char *nodeName () const { return "media:content"; }
+    PlayType playType ();
+    void closed ();
 };
 
 } //namespace ATOM
