@@ -146,7 +146,7 @@ void Control::init () {
     if (stat (tmp, &st))
         tmp = "/usr/share/applications/kmplayer/playlist.xml";
     pldoc->path = tmp;
-    m_app->playlists_tree_id = addTree (m_app, playlists,
+    m_app->playlists_tree_id = addTree (m_app, pldoc,
             gtk_icon_theme_load_icon(icon_theme,
                 "kmplayer-fav", 26, (GtkIconLookupFlags)0, 0L));
     GConfClient *client = (GConfClient *)(GConf &) *m_config;
@@ -159,7 +159,7 @@ void Control::init () {
         if (str && *str) {
             Playlist *expl = new Playlist (m_app, 0L);
             extra_playlists.append (expl);
-            expl->caption = entry->key + strlen (GCONF_KEY_LISTS) + 1;
+            expl->title = entry->key + strlen (GCONF_KEY_LISTS) + 1;
             expl->path = str;
             addTree (m_app, expl,
                     gtk_icon_theme_load_icon(icon_theme,
@@ -169,7 +169,7 @@ void Control::init () {
     }
     g_slist_free (plists);
     recents = new KMPlayer::Recents (m_app);
-    m_app->recents_tree_id = addTree (m_app, recents,
+    m_app->recents_tree_id = addTree (m_app, recents->mrl (),
             gtk_icon_theme_load_icon(icon_theme,
                 "qgn_list_gene_url_history", 26, (GtkIconLookupFlags)0, 0L));
     String sf = m_config->readEntry (GCONF_KEY_SAVE_FOLDER, String());
@@ -272,7 +272,7 @@ void Control::stop (bool keep_fullscreen) {
     }
     VIEWAREA(m_app)->reset ();
 
-    NodePtr recent = recents->firstChild ();
+    Node *recent = recents->firstChild ();
     Mrl *mrl = m_process->mrl ();
     if (recent && mrl &&
             m_process->isLocalFile () &&
@@ -331,7 +331,7 @@ void Control::openDocument (NodePtr doc, NodePtr cur) {
     doc->document()->notify_listener = this;
     playlist_data->document = doc;
     playlist_data->current = cur ? cur : doc;
-    cur_url = doc->caption;
+    cur_url = doc->caption ();
     app_set_title ((const char *) cur_url, m_app);
     back_request = cur;
     g_timeout_add (0, playSchedule, static_cast <void *> (this));
@@ -358,7 +358,7 @@ void Control::playCurrent () {
         if (!playlist_data->current)
             setCurrent (playlist_data->document);
         else { // ugly code duplicate w/ back_request
-            for (NodePtr p = playlist_data->current->parentNode(); p; p = p->parentNode())
+            for (Node *p = playlist_data->current->parentNode(); p; p = p->parentNode())
                 p->setState (Element::state_activated);
         }
         playlist_data->current->activate ();
@@ -413,7 +413,7 @@ bool Control::applyBackRequest() {
         n->activate();
     } else {
         setCurrent (back_request);
-        for (NodePtr p = back_request->parentNode(); p; p = p->parentNode())
+        for (Node *p = back_request->parentNode(); p; p = p->parentNode())
             p->setState (Element::state_activated);
         back_request = 0L;
         g_timeout_add(0, playCurrentSchedule, static_cast<void*>(this));
@@ -574,7 +574,7 @@ bool Control::setCurrent (NodePtr n) {
                 break;
             if (mrl->view_mode == Mrl::SingleMode)
                 url = mrl->absolutePath ();
-            NodePtr p = mrl->parentNode ();
+            Node *p = mrl->parentNode ();
             for (mrl = 0L; p; p = p->parentNode ())
                 if (p->mrl ()) {
                     mrl = p->mrl ();
@@ -761,16 +761,16 @@ void Control::addRecent (const String &url) {
     if (url == "Playlist://")
         return;
     recents->defer ();
-    NodePtr c = recents->firstChild ();
+    Node *c = recents->firstChild ();
     if (!c || c->mrl()->src != url) {
         recents->insertBefore (new Recent (recents, m_app, url), c);
         c = recents->firstChild ()->nextSibling ();
         int count = 1;
-        KMPlayer::NodePtr more;
+        Node *more = NULL;
         while (c) {
             if (c->id == id_node_recent_node &&
                     c->mrl ()->src == url) {
-                KMPlayer::NodePtr tmp = c->nextSibling ();
+                Node *tmp = c->nextSibling ();
                 recents->removeChild (c);
                 c = tmp;
             } else {
@@ -787,9 +787,9 @@ void Control::addRecent (const String &url) {
         if (more) {
             KMPlayer::NodePtr item;
             if (count > 10) {
-                KMPlayer::NodePtr item = more->previousSibling ();
+                NodePtr item = more->previousSibling ();
                 recents->removeChild (item);
-                more->insertBefore (item, more->firstChild ());
+                more->insertBefore (item.ptr (), more->firstChild ());
             }
             if (more->firstChild ())
                 c = more->firstChild ()->nextSibling ();
@@ -797,7 +797,7 @@ void Control::addRecent (const String &url) {
             while (c) {
                 if (c->id == id_node_recent_node &&
                         c->mrl ()->src == url) {
-                    KMPlayer::NodePtr tmp = c->nextSibling ();
+                    Node *tmp = c->nextSibling ();
                     more->removeChild (c);
                     c = tmp;
                 } else {
@@ -915,7 +915,7 @@ static Mrl *findFirstPlayable (Node *n) {
     if (n) {
         if (n->isPlayable ())
             return n->mrl ();
-        for (Node *c = n->firstChild().ptr (); c; c = c->nextSibling().ptr ()) {
+        for (Node *c = n->firstChild (); c; c = c->nextSibling ()) {
             Mrl *mrl = findFirstPlayable (c);
             if (mrl)
                 return mrl;
@@ -1025,7 +1025,7 @@ void cb_favorites_add (Application *app) {
                     n = mrl;
                 } else {
                     n = new PlaylistItem (doc, app, false, mrl->src);
-                    n->caption = mrl->caption;
+                    n->setCaption (mrl->title);
                 }
                 String xml = n->outerXML ();
                 TextStream inxml ((const char *) xml);
