@@ -67,14 +67,7 @@ Node * KMPlayer::fromXMLDocumentTag (NodePtr & d, const String & tag) {
 
 //-----------------------------------------------------------------------------
 
-namespace {
-    struct XMLStringlet {
-        const String str;
-        XMLStringlet (const String & s) : str (s) {}
-    };
-} // namespace
-
-TextStream & operator << (TextStream & out, const XMLStringlet & txt) {
+TextStream &KMPlayer::operator << (TextStream &out, const XMLStringlet &txt) {
     int len = int (txt.str.length ());
     for (int i = 0; i < len; ++i) {
         if (txt.str [i] == Char ('<')) {
@@ -124,7 +117,7 @@ KDE_NO_CDTOR_EXPORT TimerPosting::TimerPosting (int ms, unsigned eid)
 
 KDE_NO_CDTOR_EXPORT Node::Node (NodePtr & d, short _id)
  : m_doc (d), state (state_init), id (_id),
-   auxiliary_node (false), editable (true) {}
+   auxiliary_node (false), editable (true), open (false) {}
 
 Node::~Node () {
     clear ();
@@ -155,6 +148,17 @@ bool Node::expose () const {
     return true;
 }
 
+String Node::caption () const {
+    return String ();
+}
+
+void Node::setCaption (const String &) {
+}
+
+String Node::description () const {
+    return String ();
+}
+
 void Node::activate () {
     //kdDebug () << nodeName () << " Node::activate" << endl;
     setState (state_activated);
@@ -252,7 +256,7 @@ void Node::appendChild (NodePtr c) {
     TreeNode<Node>::appendChild (c);
 }
 
-KDE_NO_EXPORT void Node::insertBefore (NodePtr c, NodePtr b) {
+KDE_NO_EXPORT void Node::insertBefore (Node *c, Node *b) {
     ASSERT (!c->parentNode ());
     document()->m_tree_version++;
     TreeNode<Node>::insertBefore (c, b);
@@ -298,23 +302,23 @@ KDE_NO_EXPORT void Node::characterData (const String & s) {
 }
 
 void Node::normalize () {
-    NodePtr e = firstChild ();
+    Node *e = firstChild ();
     while (e) {
-        NodePtr tmp = e->nextSibling ();
+        Node *tmp = e->nextSibling ();
         if (!e->isElementNode () && e->id == id_node_text) {
             String val = e->nodeValue (); //.simplifyWhiteSpace ();
             if (val.isEmpty ())
                 removeChild (e);
             else
-                convertNode <TextNode> (e)->setText (val);
+                static_cast <TextNode *> (e)->setText (val);
         } else
             e->normalize ();
         e = tmp;
     }
 }
 
-static void getInnerText (const NodePtr p, TextStream & out) {
-    for (NodePtr e = p->firstChild (); e; e = e->nextSibling ()) {
+static void getInnerText (const Node *p, TextStream & out) {
+    for (Node *e = p->firstChild (); e; e = e->nextSibling ()) {
         if (e->id == id_node_text || e->id == id_node_cdata)
             out << e->nodeValue () << Char ('\n');
         else
@@ -324,26 +328,26 @@ static void getInnerText (const NodePtr p, TextStream & out) {
 
 String Node::innerText () const {
     TextStream out;
-    getInnerText (m_self, out);
+    getInnerText (this, out);
     return out.release ();
 }
 
-static void getOuterXML (const NodePtr p, TextStream & out, int depth) {
+static void getOuterXML (const Node *p, TextStream & out, int depth) {
     if (!p->isElementNode ()) { // #text or #cdata
         if (p->id == id_node_cdata)
             out << "<![CDATA[" << p->nodeValue () << "]]>" << Char ('\n');
         else
             out << XMLStringlet (p->nodeValue ()) << Char ('\n');
     } else {
-        Element * e = convertNode <Element> (p);
+        const Element *e = static_cast <const Element *> (p);
         String indent (String ().fill (Char (' '), depth));
         out << indent << Char ('<') << XMLStringlet (e->nodeName ());
-        for (AttributePtr a = e->attributes()->first(); a; a = a->nextSibling())
+        for (Attribute *a = e->attributes()->first(); a; a = a->nextSibling())
             out << " " << XMLStringlet (a->name ().toString ()) <<
                 "=\"" << XMLStringlet (a->value ()) << "\"";
         if (e->hasChildNodes ()) {
             out << Char ('>') << Char ('\n');
-            for (NodePtr c = e->firstChild (); c; c = c->nextSibling ())
+            for (Node *c = e->firstChild (); c; c = c->nextSibling ())
                 getOuterXML (c, out, depth + 1);
             out << indent << String("</") << XMLStringlet (e->nodeName()) <<
                 Char ('>') << Char ('\n');
@@ -354,14 +358,14 @@ static void getOuterXML (const NodePtr p, TextStream & out, int depth) {
 
 String Node::innerXML () const {
     TextStream out;
-    for (NodePtr e = firstChild (); e; e = e->nextSibling ())
+    for (Node *e = firstChild (); e; e = e->nextSibling ())
         getOuterXML (e, out, 0);
     return out.release ();
 }
 
 String Node::outerXML () const {
     TextStream out;
-    getOuterXML (m_self, out, 0);
+    getOuterXML (this, out, 0);
     return out.release ();
 }
 
@@ -369,9 +373,13 @@ Node::PlayType Node::playType () {
     return play_type_none;
 }
 
-void Node::opened () {}
+void Node::opened () {
+    open = true;
+}
 
-void Node::closed () {}
+void Node::closed () {
+    open = false;
+}
 
 void *Node::message (MessageType msg, void *content) {
     if (MsgChildFinished == msg) {
@@ -461,12 +469,11 @@ Element::~Element () {
     delete d;
 }
 
-void Element::setParam (const TrieString &param, const String &val, int *mid) {
-    ParamValue * pv = d->params [param];
+void Element::setParam (const TrieString &name, const String &val, int *mid) {
+    ParamValue * pv = d->params [name];
     if (!pv) {
-        pv = new ParamValue (mid ? String () : val);
-        //d->params.insert (std::make_pair (param, pv));
-        d->params [param] = pv;
+        pv = new ParamValue (mid ? getAttribute (name) : val);
+        d->params [name] = pv;
     }
     if (mid) {
         if (!pv->modifications)
@@ -477,20 +484,21 @@ void Element::setParam (const TrieString &param, const String &val, int *mid) {
             *mid = pv->modifications->size ();
             pv->modifications->push_back (val);
         }
-    } else
+    } else {
         pv->setValue (val);
-    parseParam (param, val);
+    }
+    parseParam (name, val);
 }
 
 String Element::param (const TrieString & name) {
     ParamValue * pv = d->params [name];
     if (pv)
         return pv->value ();
-    return String ();
+    return getAttribute (name);
 }
 
-void Element::resetParam (const TrieString & param, int mid) {
-    ParamValue * pv = d->params [param];
+void Element::resetParam (const TrieString &name, int mid) {
+    ParamValue * pv = d->params [name];
     if (pv && pv->modifications) {
         if (int (pv->modifications->size ()) > mid && mid > -1) {
             (*pv->modifications) [mid].clear ();
@@ -502,28 +510,31 @@ void Element::resetParam (const TrieString & param, int mid) {
         if (pv->modifications->size () == 0) {
             delete pv->modifications;
             pv->modifications = 0L;
-            val = pv->value ();
             if (val.isNull ()) {
                 delete pv;
-                d->params.erase (param);
+                d->params.erase (name);
             }
         }
-        parseParam (param, val);
+        parseParam (name, val);
     } else
-        errorLog () << "resetting " << param.toString() << " that doesn't exists" << endl;
+        errorLog () << "resetting " << name.toString() << " that doesn't exists" << endl;
 }
 
 void Element::setAttribute (const TrieString & name, const String & value) {
-    for (AttributePtr a = m_attributes->first (); a; a = a->nextSibling ())
+    for (Attribute *a = m_attributes->first (); a; a = a->nextSibling ())
         if (name == a->name ()) {
-            a->setValue (value);
+            if (value.isNull ())
+                m_attributes->remove (a);
+            else
+                a->setValue (value);
             return;
         }
-    m_attributes->append (new Attribute (name, value));
+    if (!value.isNull ())
+        m_attributes->append (new Attribute (name, value));
 }
 
 String Element::getAttribute (const TrieString & name) {
-    for (AttributePtr a = m_attributes->first (); a; a = a->nextSibling ())
+    for (Attribute *a = m_attributes->first (); a; a = a->nextSibling ())
         if (name == a->name ())
             return a->value ();
     return String ();
@@ -531,8 +542,8 @@ String Element::getAttribute (const TrieString & name) {
 
 void Element::init () {
     d->clear();
-    for (AttributePtr a = attributes ()->first (); a; a = a->nextSibling ())
-        setParam (a->name (), a->value ());
+    for (Attribute *a = attributes ()->first (); a; a = a->nextSibling ())
+        parseParam (a->name (), a->value ());
 }
 
 void Element::reset () {
@@ -569,15 +580,25 @@ void Attribute::setValue (const String & v) {
 
 //-----------------------------------------------------------------------------
 
+String Title::caption () const {
+    return title;
+}
+
+void Title::setCaption (const String &t) {
+    title = t;
+}
+
+//-----------------------------------------------------------------------------
+
 static bool hasMrlChildren (const NodePtr & e) {
-    for (NodePtr c = e->firstChild (); c; c = c->nextSibling ())
+    for (Node *c = e->firstChild (); c; c = c->nextSibling ())
         if (c->isPlayable () || hasMrlChildren (c))
             return true;
     return false;
 }
 
 Mrl::Mrl (NodePtr & d, short id)
-    : Element (d, id), cached_ismrl_version (~0),
+    : Title (d, id), cached_ismrl_version (~0),
       media_info (NULL),
       aspect (0), repeat (0),
       clip_begin (0),
@@ -605,8 +626,8 @@ String Mrl::absolutePath () {
         return src;
     String path = src;
     if (!path.isEmpty() && !path.startsWith ("tv:/")) {
-        for (NodePtr e = parentNode (); e; e = e->parentNode ()) {
-            Mrl * mrl = e->mrl ();
+        for (Node *e = parentNode (); e; e = e->parentNode ()) {
+            Mrl *mrl = e->mrl ();
             if (mrl && !mrl->src.isEmpty () && mrl->src != src) {
                 path = URL (mrl->absolutePath (), path).url ();
                 break;
@@ -655,7 +676,7 @@ void *Mrl::message (MessageType msg, void *content) {
         return NULL;
 
     case MsgQueryRoleChildDisplay:
-        for (NodePtr p = parentNode (); p; p = p->parentNode ())
+        for (Node *p = parentNode (); p; p = p->parentNode ())
             if (p->mrl ())
                 return p->message (msg, content);
         return NULL;
@@ -812,13 +833,13 @@ Document::~Document () {
 }
 
 static Node *getElementByIdImpl (Node *n, const String & id, bool inter) {
-    NodePtr elm;
+    Node *elm = NULL;
     if (!n->isElementNode ())
         return NULL;
     Element *e = static_cast <Element *> (n);
     if (e->getAttribute (StringPool::attr_id) == id)
         return n;
-    for (Node *c = e->firstChild ().ptr (); c; c = c->nextSibling ().ptr ()) {
+    for (Node *c = e->firstChild (); c; c = c->nextSibling ()) {
         if (!inter && c->mrl () && c->mrl ()->opener.ptr () == n)
             continue;
         if ((elm = getElementByIdImpl (c, id, inter)))
@@ -1211,7 +1232,7 @@ GenericURL::GenericURL (NodePtr & d, const String & s, const String & name)
     src = s;
     if (!src.isEmpty ())
         setAttribute (StringPool::attr_src, src);
-    caption = name;
+    title = name;
     if (!name.isEmpty ())
         setAttribute (StringPool::attr_title, name);
 }
@@ -1219,6 +1240,7 @@ GenericURL::GenericURL (NodePtr & d, const String & s, const String & name)
 KDE_NO_EXPORT void GenericURL::closed () {
     if (src.isEmpty ())
         src = getAttribute (StringPool::attr_src);
+    Mrl::closed ();
 }
 
 //-----------------------------------------------------------------------------
@@ -1228,7 +1250,7 @@ GenericMrl::GenericMrl (NodePtr & d, const String & s, const String & name, cons
     src = s;
     if (!src.isEmpty ())
         setAttribute (StringPool::attr_src, src);
-    caption = name;
+    title = name;
     if (!name.isEmpty ())
         setAttribute (StringPool::attr_name, name);
 }
@@ -1239,12 +1261,13 @@ void GenericMrl::closed () {
         if (src.isEmpty ())
             src = getAttribute (StringPool::attr_url);
     }
-    if (caption.isEmpty ())
-        caption = getAttribute (StringPool::attr_name);
+    if (title.isEmpty ())
+        title = getAttribute (StringPool::attr_name);
+    Mrl::closed ();
 }
 
 bool GenericMrl::expose () const {
-    return !caption.isEmpty () || //return false if no caption and only one
+    return !title.isEmpty () || //return false if no caption and only one
         previousSibling () || nextSibling ();
 }
 
@@ -1529,8 +1552,11 @@ void KMPlayer::readXML (NodePtr root, TextStream & in, const String & firstline,
     }
     if (!in.atEnd ())
         parser.parse (in);
-    for (NodePtr e = root; e; e = e->parentNode ())
+    for (NodePtr e = root; e; e = e->parentNode ()) {
+        if (e->open)
+            break;
         e->closed ();
+    }
     //doc->normalize ();
     //kdDebug () << root->outerXML ();
 }
