@@ -116,14 +116,14 @@ public:
         : m_first (f), m_last (l) {}
     ~List () { clear (); }
 
-    typename Item<T>::SharedType first () const { return m_first; }
-    typename Item<T>::SharedType last () const { return m_last; }
+    T* first () const { return m_first.ptr (); }
+    T* last () const { return m_last.ptr (); }
     void append (typename Item<T>::SharedType c);
-    void insertBefore(typename Item<T>::SharedType c, typename Item<T>::SharedType b);
+    void insertBefore (typename Item<T>::SharedType c, typename Item<T>::SharedType b);
     void remove (typename Item<T>::SharedType c);
     void clear ();
     unsigned int length () const;
-    typename Item<T>::SharedType item (int i) const;
+    T* item (int i) const;
 protected:
     typename Item<T>::SharedType m_first;
     typename Item<T>::WeakType m_last;
@@ -139,8 +139,8 @@ class KMPLAYER_EXPORT ListNodeBase : public Item <T> {
 public:
     virtual ~ListNodeBase () {}
 
-    typename Item<T>::SharedType nextSibling () const { return m_next; }
-    typename Item<T>::SharedType previousSibling () const { return m_prev; }
+    T* nextSibling () const { return m_next.ptr (); }
+    T* previousSibling () const { return m_prev.ptr (); }
 protected:
     ListNodeBase () {}
     typename Item<T>::SharedType m_next;
@@ -166,14 +166,14 @@ class KMPLAYER_EXPORT TreeNode : public ListNodeBase <T> {
 public:
     virtual ~TreeNode () {}
 
-    virtual void insertBefore (typename Item<T>::SharedType c, typename Item<T>::SharedType b);
+    virtual void insertBefore (T *c, T *b);
     virtual void appendChild (typename Item<T>::SharedType c);
     virtual void removeChild (typename Item<T>::SharedType c);
 
     bool hasChildNodes () const { return m_first_child != 0L; }
-    typename Item<T>::SharedType parentNode () const { return m_parent; }
-    typename Item<T>::SharedType firstChild () const { return m_first_child; }
-    typename Item<T>::SharedType lastChild () const { return m_last_child; }
+    T* parentNode () const { return m_parent.ptr (); }
+    T* firstChild () const { return m_first_child.ptr (); }
+    T* lastChild () const { return m_last_child.ptr (); }
 
 protected:
     TreeNode () {}
@@ -292,6 +292,13 @@ private:
 
 typedef SharedPtr <Connection> ConnectionPtr;
 
+struct XMLStringlet {
+    const String str;
+    XMLStringlet (const String & s) : str (s) {}
+};
+
+TextStream & operator << (TextStream &out, const XMLStringlet &txt);
+
 /*
  * Base class for XML nodes. Provides a w3c's DOM like API
  *
@@ -329,18 +336,20 @@ public:
     String outerXML () const;
     virtual const char * nodeName () const;
     virtual String nodeValue () const;
-    virtual void setNodeName (const String &) {}
 
     /**
      * If this is a derived Mrl object and has a SRC attribute
      */
     virtual PlayType playType ();
     bool isPlayable () { return playType () > play_type_none; }
-    virtual bool isElementNode () { return false; }
+    virtual bool isElementNode () const { return false; }
     /**
      * If this node should be visible to the user
      */
     virtual bool expose () const;
+    virtual String caption () const;
+    virtual void setCaption (const String &t);
+    virtual String description () const;
     /**
      * If this node should be visible to the user
      */
@@ -415,7 +424,7 @@ public:
     virtual void clear ();
     void clearChildren ();
     void appendChild (NodePtr c);
-    void insertBefore (NodePtr c, NodePtr b);
+    void insertBefore (Node *c, Node *b);
     void removeChild (NodePtr c);
     void replaceChild (NodePtr _new, NodePtr old);
     /*
@@ -438,13 +447,11 @@ protected:
     Node (NodePtr & d, short _id=0);
     NodePtr m_doc;
 public:
-    String caption;
     State state;
     short id;
-private:
-    bool auxiliary_node;
-protected:
-    bool editable;
+    bool auxiliary_node : 1;
+    bool editable : 1;
+    bool open : 1;
 };
 
 ITEM_AS_POINTER(KMPlayer::Node)
@@ -458,6 +465,7 @@ const short id_node_group_node = 25;
 const short id_node_playlist_document = 26;
 const short id_node_playlist_item = 27;
 const short id_node_param = 28;
+const short id_node_svg = 29;
 
 /*
  * Element node, XML node that can have attributes
@@ -468,11 +476,11 @@ public:
     void setAttributes (AttributeListPtr attrs);
     void setAttribute (const TrieString & name, const String & value);
     String getAttribute (const TrieString & name);
-    KDE_NO_EXPORT AttributeListPtr attributes () const { return m_attributes; }
+    KDE_NO_EXPORT AttributeList *attributes () const { return m_attributes.ptr (); }
     virtual void init ();
     virtual void reset ();
     virtual void clear ();
-    virtual bool isElementNode () { return true; }
+    virtual bool isElementNode () const { return true; }
     virtual void accept (Visitor * v);
     /**
      * Params are like attributes, but meant to be set dynamically. Caller may
@@ -493,27 +501,34 @@ private:
     ElementPrivate * d;
 };
 
-/**
- * Node that references another node
- */
-class RefNode : public Node {
-public:
-    RefNode (NodePtr &doc, NodePtr ref) : Node (doc), ref_node (ref) {}
-    virtual const char *nodeName () const { return "#ref"; }
-    NodePtr refNode () const { return ref_node; }
-protected:
-    NodePtrW ref_node;
-};
-
 template <class T>
 inline KDE_NO_EXPORT T * convertNode (NodePtr e) {
     return static_cast <T *> (e.ptr ());
 }
 
+KMPLAYER_NO_EXPORT
+inline Node *findChildWithId (const Node *p, const short id) {
+    for (Node *c = p->firstChild (); c; c = c->nextSibling ())
+        if (id == c->id)
+            return c;
+    return NULL;
+}
+
+class KMPLAYER_EXPORT Title : public Element {
+protected:
+    Title (NodePtr & d, short id=0) : Element (d, id ) {}
+
+public:
+    virtual String caption () const;
+    virtual void setCaption (const String &t);
+
+    String title;
+};
+
 /**
  * Element representing a playable link, like URL to a movie or playlist.
  */
-class KMPLAYER_EXPORT Mrl : public Element {
+class KMPLAYER_EXPORT Mrl : public Title {
 protected:
     Mrl (NodePtr & d, short id=0);
     Node *childFromTag (const String & tag);
@@ -687,7 +702,7 @@ public:
 
     void timeOfDay (struct timeval &);
     PostponePtr postpone ();
-    bool postponed () const { return !!postpone_ref; }
+    bool postponed () const { return !!postpone_ref || !! postpone_lock; }
     /**
      * Called by PlayListNotify, processes events in event_queue with timeout set to now
      */
@@ -915,10 +930,10 @@ template <class T> inline void List<T>::clear () {
 }
 
 template <class T>
-inline typename Item<T>::SharedType List<T>::item (int i) const {
+inline T* List<T>::item (int i) const {
     for (typename Item<T>::SharedType t = m_first; t; t = t->nextSibling(), --i)
         if (i == 0)
-            return t;
+            return t.ptr ();
     return typename Item<T>::SharedType ();
 }
 
@@ -935,20 +950,19 @@ inline void TreeNode<T>::appendChild (typename Item<T>::SharedType c) {
 }
 
 template <class T>
-inline void TreeNode<T>::insertBefore (
-        typename Item<T>::SharedType c, typename Item<T>::SharedType b) {
+inline void TreeNode<T>::insertBefore (T *c, T *b) {
     if (!b) {
-        appendChild (c);
+        appendChild (c->m_self);
     } else {
+        c->m_next = b->m_self;
         if (b->m_prev) {
-            b->m_prev->m_next = c;
+            b->m_prev->m_next = c->m_self;
             c->m_prev = b->m_prev;
         } else {
             c->m_prev = 0L;
-            m_first_child = c;
+            m_first_child = c->m_self;
         }
-        b->m_prev = c;
-        c->m_next = b;
+        b->m_prev = c->m_self;
         c->m_parent = Item<T>::m_self;
     }
 }
