@@ -95,16 +95,16 @@ static GdkPixbuf *iconForNode (Node *n, int child_count) {
 
 static
 int populate (GtkTreeStore * tree_store, PlayListData * pld,
-        NodePtr e, GtkTreeIter * riter, GtkTreeIter * piter,
+        Node *e, GtkTreeIter * riter, GtkTreeIter * piter,
         GtkTreePath ** fpath)
 {
     GtkTreeIter iter, citer, cciter;
     bool m_have_dark_nodes = false;
     int child_count = 0;
     if (!pld->show_all && !e->expose () && piter) {
-        if (pld->current == e)
+        if (pld->current.ptr () == e)
             *fpath = gtk_tree_model_get_path (GTK_TREE_MODEL (tree_store), piter);
-        for (NodePtr c = e->firstChild (); c; c = c->nextSibling ())
+        for (Node *c = e->firstChild (); c; c = c->nextSibling ())
             child_count += populate (tree_store, pld, c, riter, piter, fpath);
         return child_count;
     }
@@ -112,32 +112,33 @@ int populate (GtkTreeStore * tree_store, PlayListData * pld,
         gtk_tree_store_append (tree_store, &iter, piter);
     else
         iter = *riter;
-    Mrl *mrl = e->mrl ();
-    String text(e->caption.isEmpty() ? e->nodeName() : (const char*)e->caption);
-    if (mrl && !pld->show_all) {
-        if (mrl->caption.isEmpty ()) {
+    String text (e->caption ());
+    if (text.isEmpty ()) {
+        Mrl *mrl = e->mrl ();
+        text = e->nodeName ();
+        if (mrl && !pld->show_all) {
             if (!mrl->src.isEmpty())
                 text = mrl->src; // URL(mrl->src).url (); //prettyURL();
             else if (e->isDocument ())
                 text = e->hasChildNodes () ? "unnamed" : "None";
+        } else if (e->id == id_node_text) {
+            String txt (e->nodeValue ());
+            int pos = txt.indexOf (Char ('\n'));
+            if (pos > -1) {
+                txt.truncate (pos);
+                txt += String (" ..");
+            }
+            text = txt;
         }
-    } else if (e->id == id_node_text) {
-        String txt (e->nodeValue ());
-        int pos = txt.indexOf (Char ('\n'));
-        if (pos > -1) {
-            txt.truncate (pos);
-            txt += String (" ..");
-        }
-        text = txt;
     }
     gtk_tree_store_set (tree_store, &iter,
             COL_TEXT, (const char *) text, COL_ID, pld->id, -1);
-    if (pld->current == e)
+    if (pld->current.ptr () == e)
         *fpath = gtk_tree_model_get_path (GTK_TREE_MODEL (tree_store), &iter);
     //if (e->active ())
     //    ; //ensureItemVisible (item);
     if (e->isElementNode ()) {
-        AttributePtr a = convertNode<Element> (e)->attributes ()->first ();
+        Attribute *a = convertNode<Element> (e)->attributes ()->first ();
         if (a) {
             m_have_dark_nodes = true;
             if (pld->show_all) {
@@ -161,9 +162,9 @@ int populate (GtkTreeStore * tree_store, PlayListData * pld,
             }
         }
     }
-    for (NodePtr c = e->firstChild (); c; c = c->nextSibling ())
+    for (Node *c = e->firstChild (); c; c = c->nextSibling ())
         child_count += populate (tree_store, pld, c, riter, &iter, fpath);
-    GdkPixbuf *icon = piter ? iconForNode (e.ptr (), child_count) : pld->icon;
+    GdkPixbuf *icon = piter ? iconForNode (e, child_count) : pld->icon;
     gtk_tree_store_set (tree_store, &iter, COL_ICON, icon, -1);
     return child_count + 1;
 }
@@ -182,7 +183,7 @@ static gboolean findNode (PlayListData *pld, GtkTreeModel *model, Node *node, Gt
         if (!findNode (pld, model, p, iter))
             return false;
         bool first_exposed = false;
-        for (Node *n=p->firstChild().ptr(); n; n=n->nextSibling().ptr()) {
+        for (Node *n = p->firstChild (); n; n = n->nextSibling ()) {
             if (pld->show_all || n->expose ()) {
                 if (!first_exposed) {
                     first_exposed = true;
@@ -215,7 +216,7 @@ void KMPlayer::updatePlayTree (Application *app) {
     Node *node = pld->current.ptr ();
     if (!pld->show_all)
         while (node && !node->expose ())
-            node = node->parentNode ().ptr ();
+            node = node->parentNode ();
     if (!node) {
         // undo selection
         return;
@@ -283,7 +284,7 @@ void KMPlayer::updateTree (Application * app, short id) {
             gtk_tree_store_append (GTK_TREE_STORE(model), &riter, 0L);
         rpath = gtk_tree_model_get_path (model, &riter);
         populate (GTK_TREE_STORE (model), pld,
-                pld->document, &riter, 0L, &fpath);
+                pld->document.ptr (), &riter, 0L, &fpath);
         if (!fpath)
             fpath = rpath;
         else
@@ -361,21 +362,21 @@ void cb_treerow_expanded (GtkTreeView *treeview, GtkTreeIter *arg1, GtkTreePath
 }
 //gboolean    gtk_tree_view_row_expanded      (GtkTreeView *tree_view,
 //                                             GtkTreePath *path);
-static NodePtr findFirstLevelExposed (NodePtr n, int & idx) {
+static Node *findFirstLevelExposed (Node *n, int & idx) {
     if (!n)
         return n;
     if (n->expose ()) {
         if (--idx < 0)
             return n;
     } else { // not exposed node, check deeper ..
-        NodePtr c = findFirstLevelExposed (n->firstChild (), idx);
+        Node *c = findFirstLevelExposed (n->firstChild (), idx);
         while (c) {
             if (idx < 0)
                 return c;
-            NodePtr c2 = c;
+            Node *c2 = c;
             c = findFirstLevelExposed (c->nextSibling (), idx);
             while (!c) { // backtracking ..
-                NodePtr p = c2->parentNode ();
+                Node *p = c2->parentNode ();
                 while (!c && p && p != n) {
                     if (p->nextSibling ())
                         c = p->nextSibling ();
@@ -388,8 +389,8 @@ static NodePtr findFirstLevelExposed (NodePtr n, int & idx) {
     return findFirstLevelExposed (n->nextSibling (), idx);
 }
 
-static NodePtr getNode (PlayListData * pld, GtkTreePath * path) {
-    NodePtr node = pld->document;
+static Node *getNode (PlayListData * pld, GtkTreePath * path) {
+    Node *node = pld->document;
     gint depth = gtk_tree_path_get_depth (path);
     gint * idxs = gtk_tree_path_get_indices (path);
     if (pld->show_all) {
@@ -422,9 +423,9 @@ static void restoreEditMode (Application * app, PlayListData * pld) {
 
 static void setInfoContent (Application *app, Node *node, bool edit_mode) {
     app_set_info_content (app, node
-            ? (edit_mode
+            ? edit_mode
                 ? node->innerXML ()
-                : node->isPlayable () ? node->innerText () : "")
+                : node->description ()
             : "", edit_mode);
 }
 
@@ -453,15 +454,15 @@ void cb_selection_changed (GtkTreeSelection *selection, Application * app) {
         restoreEditMode (app, pld);
         GtkTreePath* path = gtk_tree_model_get_path (model, &iter);
         gtk_tree_view_scroll_to_cell (treeview, path, NULL, false, 0, 0);
-        NodePtr node = getNode (pld, path);
+        Node *node = getNode (pld, path);
         Mrl *mrl = node ? node->mrl () : NULL;
         gtk_tree_path_free (path);
-        if (mrl && pld->id > 0 && node == pld->document && !mrl->resolved)
+        if (mrl && pld->id > 0 && node == pld->document.ptr() && !mrl->resolved)
             mrl->activate ();
         if (mrl && pld->updated_document_version !=
                 mrl->document ()->m_tree_version)
             g_idle_add ((GSourceFunc) delayedUpdateTree, (gpointer) app);
-        setInfoContent (app, node.ptr (), pld->edit_mode);
+        setInfoContent (app, node, pld->edit_mode);
         CONTROL(app)->setCurrentSelected (node);
         if (mrl && pld->id > 0) //FIXME, hack to become less jumpy when changing trees
             pld->current = node;
@@ -481,17 +482,17 @@ static gboolean delayedJump (Application *app) {
 void cb_row_activated (GtkTreeView * tree_view, GtkTreePath *path,
         GtkTreeViewColumn *arg2, Application * app) {
     PlayListData *pld = getListData (CONTROL(app), path);
-    NodePtr node = getNode (pld, path);
+    Node *node = getNode (pld, path);
     debugLog() << "row activated" << endl;
     if (gtk_tree_path_get_depth (path) != 1) {
         if (pld->id == 0 || pld->edit_mode)
-            setInfoContent (app, node.ptr (), pld->edit_mode);
+            setInfoContent (app, node, pld->edit_mode);
         if (!pld->edit_mode && (pld->id == 0 || (node && node->mrl ()))) {
             if (pld->id > 0) {
                 pld->current = node;
                 g_idle_add ((GSourceFunc) delayedJump, (gpointer) app);
             } else {
-                jump (node.ptr (), CONTROL(app));
+                jump (node, CONTROL(app));
             }
         }
     }
@@ -503,7 +504,7 @@ void KMPlayer::rowSelected (Application *app) {
     GtkTreeSelection *sel = gtk_tree_view_get_selection (view);
     GtkTreeModel *model = gtk_tree_view_get_model (view);
     bool expand_collapse = false;
-    NodePtr node;
+    Node *node = NULL;
 
     debugLog() << "row selected" << endl;
     if (!gtk_tree_selection_get_selected (sel, 0L, &iter))
@@ -537,10 +538,10 @@ void KMPlayer::rowSelected (Application *app) {
                 gtk_tree_view_collapse_row (view, path);
             }
         }
-    } else {
-        setInfoContent (app, node.ptr (), pld->edit_mode);
+    } else if (node) {
+        setInfoContent (app, node, pld->edit_mode);
         if (!pld->edit_mode && (pld->id == 0 || node->mrl ()))
-            jump (node.ptr (), CONTROL(app));
+            jump (node, CONTROL(app));
     }
     gtk_tree_path_free (path);
 }
@@ -626,7 +627,7 @@ void cb_treeCellDataFunc (GtkTreeViewColumn *tree_column, GtkCellRenderer *cell,
     }
 }
 
-short KMPlayer::addTree (Application * app, NodePtr doc, GdkPixbuf * ico) {
+short KMPlayer::addTree (Application * app, Mrl *doc, GdkPixbuf * ico) {
     GtkTreeIter iter;
     PlayListData ** prev = &CONTROL(app)->playlist_data;
     short id = 0;
@@ -638,9 +639,9 @@ short KMPlayer::addTree (Application * app, NodePtr doc, GdkPixbuf * ico) {
     (*prev)->document = doc;
     (*prev)->icon = ico;
     (*prev)->updated_document = doc ? doc->document () : NULL;
-    const char * title = "empty";
-    if (doc && !doc->caption.isEmpty())
-        title = (const char *) doc->caption;
+    String title = "empty";
+    if (doc && !doc->title.isEmpty())
+        title = (const char *) doc->title;
     GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (app->playlist_view));
     gtk_tree_store_append (GTK_TREE_STORE (model), &iter, 0L);
     gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
@@ -731,7 +732,7 @@ void cb_playlistMenuActivate (GtkButton * button, Application * app) {
     GtkTreePath *path;
     PlayListData * pld = getListDataFromSelection (app, &path);
     if (pld) {
-        NodePtr node = getNode (pld, path);
+        Node *node = getNode (pld, path);
         if (node) {
             show_copy = node->id == id_node_recent_node ||
                 //node->id == id_node_group_node ||
@@ -760,7 +761,7 @@ void cb_playlistCopy (GtkButton * button, Application * app) {
     GtkTreePath *path;
     PlayListData * pld = getListDataFromSelection (app, &path);
     if (pld) {
-        NodePtr node = getNode (pld, path);
+        Node *node = getNode (pld, path);
         if (node && node->mrl ()) {
             CONTROL(app)->copyPasteNode = 0L;
             CONTROL(app)->copyPasteUrl = node->mrl ()->src;
@@ -772,10 +773,10 @@ void cb_playlistPaste (GtkButton * button, Application * app) {
     GtkTreePath *path;
     PlayListData * pld = getListDataFromSelection (app, &path);
     if (pld) {
-        NodePtr node = getNode (pld, path);
+        Node *node = getNode (pld, path);
         if (node && pld->id == app->playlists_tree_id) {
-            NodePtr newnode = CONTROL(app)->copyPasteNode
-                ? CONTROL(app)->copyPasteNode
+            Node *newnode = CONTROL(app)->copyPasteNode
+                ? CONTROL(app)->copyPasteNode.ptr ()
                 : new PlaylistItem(pld->document, app, false,CONTROL(app)->copyPasteUrl);
             if (gtk_tree_path_get_depth (path) == 1)
                 node->insertBefore (newnode, node->firstChild ());
@@ -815,14 +816,14 @@ void cb_playlistPasteInNewGroup (GtkButton * button, Application * app) {
     GtkTreePath *path;
     PlayListData * pld = getListDataFromSelection (app, &path);
     if (pld) {
-        NodePtr node = getNode (pld, path);
+        Node *node = getNode (pld, path);
         if (node && pld->id == app->playlists_tree_id) {
             String grp = getUserInput (app,
                     "Enter Name of the Group", "New Group");
             if (!grp.isNull ()) {
-                NodePtr newnode = new Group (pld->document, app, grp);
+                Node *newnode = new Group (pld->document, app, grp);
                 newnode->appendChild (CONTROL(app)->copyPasteNode
-                        ? CONTROL(app)->copyPasteNode
+                        ? CONTROL(app)->copyPasteNode.ptr ()
                         : new PlaylistItem (pld->document, app, false, CONTROL(app)->copyPasteUrl));
                 if (gtk_tree_path_get_depth (path) == 1)
                     node->insertBefore (newnode, node->firstChild ());
@@ -839,7 +840,7 @@ void cb_playlistCut (GtkButton * button, Application * app) {
     GtkTreePath *path;
     PlayListData * pld = getListDataFromSelection (app, &path);
     if (pld) {
-        NodePtr node = getNode (pld, path);
+        Node *node = getNode (pld, path);
         if (node &&
                 pld->id == app->playlists_tree_id &&
                 gtk_tree_path_get_depth (path) > 1) {
@@ -861,14 +862,14 @@ void cb_playlistTitle (GtkButton *button, Application *app) {
     GtkTreePath *path;
     PlayListData *pld = getListDataFromSelection (app, &path);
     if (pld) {
-        NodePtr node = getNode (pld, path);
+        Node *node = getNode (pld, path);
         if (node && pld->id == app->playlists_tree_id) {
-            String title = node->caption;
+            String title = node->caption ();
             title = getUserInput (app, "Enter new Title", title);
             if (!title.isNull ()) {
-                static_cast <Element *> (node.ptr ())->setAttribute
+                static_cast <Element *> (node)->setAttribute
                     (StringPool::attr_title, title);
-                node->caption = title;
+                node->setCaption (title);
                 node->document ()->m_tree_version++;
                 updateTree (app, app->playlists_tree_id);
             }
@@ -947,7 +948,7 @@ void cb_playlist_edit_commit (GtkButton * button, Application * app) {
     GtkTreePath *path;
     PlayListData * pld = getListDataFromSelection (app, &path);
     if (pld) {
-        NodePtr node = getNode (pld, path);
+        Node *node = getNode (pld, path);
         if (node) {
             NodePtr tmp = new GenericURL (pld->document, "foo", "bar");
             while (node->firstChild ()) {
