@@ -296,10 +296,10 @@ KMPLAYER_NO_EXPORT bool MPlayer::play (NodePtr node) {
 
     const TrieString aspect_ratio ("aspectRatio");
     bool found_attr = false;
-    for (NodePtr n = mrl; n && !found_attr; n = n->parentNode ()) {
+    for (Node *n = mrl; n && !found_attr; n = n->parentNode ()) {
         if (n->id != id_node_group_node && n->id != id_node_playlist_item)
-            break;
-        AttributePtr a = convertNode <Element> (n)->attributes ()->first ();
+            continue;
+        Attribute *a = convertNode <Element> (n)->attributes ()->first ();
         for (; a; a = a->nextSibling ()) {
             if (aspect_ratio == a->name ()) {
                 String asp = a->value ();
@@ -1438,7 +1438,7 @@ KMPLAYER_NO_EXPORT bool NpPlayer::play (NodePtr node) {
     GTree *params = g_tree_new ((GCompareFunc)::strcmp);
     Mrl *mrl = this->mrl ();
     if (mrl->id == id_node_html_object) {
-        for (Node *n = mrl->firstChild().ptr(); n; n = n->nextSibling().ptr()) {
+        for (Node *n = mrl->firstChild (); n; n = n->nextSibling ()) {
             /*if (n->id == KMPlayer::id_node_param) {
                 KMPlayer::Element *e = static_cast <KMPlayer::Element *> (n);
                 String name = e->getAttribute (KMPlayer::StringPool::attr_name);
@@ -1448,7 +1448,7 @@ KMPLAYER_NO_EXPORT bool NpPlayer::play (NodePtr node) {
                             strdup ((const char *) e->getAttribute (KMPlayer::StringPool::attr_value)));
             } else*/ if (n->id == KMPlayer::id_node_html_embed) {
                 KMPlayer::Element *e = static_cast <KMPlayer::Element *> (n);
-                AttributePtr a = e->attributes ()->first ();
+                Attribute *a = e->attributes ()->first ();
                 for (; a; a = a->nextSibling ()) {
                     String nm = a->name().toString();
                     if (!g_tree_lookup (params, (const char *) nm))
@@ -1671,10 +1671,11 @@ bool KMPLAYER_NO_EXPORT Downloader::play (NodePtr mrl) {
     saved_file.clear ();
     if (!m_url.isEmpty ()) {
         gchar *filename = NULL;
-        String name = mrl->caption;
+        String name = mrl->caption ();
         if (!name.isEmpty ()) {
+            String title = name;
             name = "";
-            for (const char *s = (const char *)mrl->caption; s && *s; ++s)
+            for (const char *s = (const char *)title; s && *s; ++s)
                 switch (*s) {
                     case '\n':
                     case ';':
