@@ -188,7 +188,7 @@ bool GLibString::startsWith (const GLibString & prefix) const {
         return g_str_has_prefix (data->ptr, prefix.data->ptr);
     return !prefix.data;
 }
-    
+
 bool GLibString::endsWith (const GLibString & suffix) const {
     if (!suffix.data && !data)
         return true;
@@ -289,6 +289,8 @@ GLibString GLibString::mid (const int index, unsigned len) const {
 GLibString GLibString::left (int index) const {
     if (!data)
         return GLibString ();
+    if (index < 0)
+        return GLibString (*this);
     gchar * off = g_utf8_offset_to_pointer (data->ptr, index);
     if (!off)
         return GLibString ();
