@@ -136,6 +136,13 @@ void ImageData::copyImage (Surface *s, int w, int h, cairo_surface_t *similar,
 
 #define USE_IMG_SURFACE 0
 
+# define CAIRO_SET_SOURCE_ARGB(cr,c)          \
+    cairo_set_source_rgba ((cr),              \
+            1.0 * (((c) >> 16) & 0xff) / 255, \
+            1.0 * (((c) >> 8) & 0xff) / 255,  \
+            1.0 * (((c)) & 0xff) / 255,       \
+            1.0 * (((c) >> 24) & 0xff) / 255)
+
 class KMPLAYER_NO_EXPORT CairoPaintVisitor : public Visitor {
     IRect clip;
     ViewArea *view_widget;
@@ -304,24 +311,38 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Layout *layout) {
     }
 }
 
+#define REGION_SCROLLBAR_WIDTH 30
+
+static void cairoDrawRect (cairo_t *cr, unsigned int color,
+        int x, int y, int w, int h) {
+    CAIRO_SET_SOURCE_ARGB (cr, color);
+    cairo_rectangle (cr, x, y, w, h);
+    cairo_fill (cr);
+}
+
 KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::RegionBase *reg) {
     Surface *s = (Surface *) reg->message (MsgQueryRoleDisplay);
     if (s) {
         SRect rect = s->bounds;
 
-        Matrix m = matrix;
         Single x = rect.x(), y = rect.y(), w = rect.width(), h = rect.height();
         matrix.getXYWH (x, y, w, h);
         if (clip.intersect (IRect (x, y, w, h)).isEmpty ())
             return;
+        Matrix m = matrix;
         matrix = Matrix (rect.x(), rect.y(), 1.0, 1.0);
         matrix.transform (m);
         IRect clip_save = clip;
         clip = clip.intersect (IRect (x, y, w, h));
         cairo_save (cr);
+
+        Surface *cs = s->firstChild ();
+        if (!s->virtual_size.isEmpty ())
+            matrix.translate (-s->x_scroll, -s->y_scroll);
+
         ImageActor *im = reg->media_info
-            ? (ImageActor *) reg->media_info->media
-            : NULL;
+             ? (ImageActor *) reg->media_info->media
+             : NULL;
         ImageData *bg_img = im && !im->isEmpty() ? im->cached_img.ptr () : NULL;
         if ((SMIL::RegionBase::ShowAlways == reg->show_background ||
                     reg->m_AttachedMediaTypes->first ()) &&
@@ -356,6 +377,80 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::RegionBase *reg) {
             cairo_restore (cr);
         }
         traverseRegion (reg, s);
+        cs = s->firstChild ();
+        if (cs && (s->scroll || cs->scroll) && cs == s->lastChild ()) {
+            SRect r = cs->bounds;
+            if (r.width () > rect.width () || r.height () > rect.height ()) {
+                if (s->virtual_size.isEmpty ())
+                    s->x_scroll = s->y_scroll = 0;
+                s->virtual_size = r.size;
+                matrix.getWH (s->virtual_size.width, s->virtual_size.height);
+                s->virtual_size.width += REGION_SCROLLBAR_WIDTH;
+                s->virtual_size.height += REGION_SCROLLBAR_WIDTH;
+                const int vy = s->virtual_size.height;
+                const int vw = s->virtual_size.width;
+                int sbw = REGION_SCROLLBAR_WIDTH;
+                int sbx = x + w - sbw;
+                int sby = y;
+                int sbh = h - REGION_SCROLLBAR_WIDTH;
+                IRect sb_clip = clip.intersect (IRect (sbx, sby, sbw, sbh));
+                if (!sb_clip.isEmpty ()) {
+                    int knob_h = sbh * (int) h / vy;
+                    int knob_y = (int) y + s->y_scroll * sbh / vy;
+                    IRect knob (sbx, knob_y, sbw, knob_h);
+                    cairo_save (cr);
+                    cairo_rectangle (cr, sb_clip.x, sb_clip.y, sb_clip.w, sb_clip.h);
+                    cairo_clip (cr);
+                    cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
+                    cairo_set_line_width (cr, 2);
+                    CAIRO_SET_SOURCE_ARGB (cr, 0x80A0A0A0);
+                    cairo_rectangle (cr, sbx + 1, sby + 1, sbw - 2, sbh - 2);
+                    cairo_stroke (cr);
+                    if (s->y_scroll)
+                        cairoDrawRect (cr, 0x80000000,
+                                sbx + 2, sby + 2,
+                                sbw - 4, knob.y - sby - 2);
+                    cairoDrawRect (cr, 0x80808080,
+                            knob.x + 2, knob.y,
+                            knob.w - 4, knob.h);
+                    if (sby + sbh - knob.y - knob.h - 2 > 0)
+                        cairoDrawRect (cr, 0x80000000,
+                                sbx + 2, knob.y + knob.h,
+                                sbw - 4, sby + sbh - knob.y - knob.h - 2);
+                    cairo_restore (cr);
+                }
+                sbh = REGION_SCROLLBAR_WIDTH;
+                sbx = x;
+                sby = y + h - sbh;
+                sbw = w - REGION_SCROLLBAR_WIDTH;
+                sb_clip = clip.intersect (IRect (sbx, sby, sbw, sbh));
+                if (!sb_clip.isEmpty ()) {
+                    int knob_w = sbw * (int) w / vw;
+                    int knob_x = (int) x + s->x_scroll * sbw / vw;
+                    IRect knob (knob_x, sby, knob_w, sbh);
+                    cairo_save (cr);
+                    cairo_rectangle (cr, sb_clip.x, sb_clip.y, sb_clip.w, sb_clip.h);
+                    cairo_clip (cr);
+                    cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
+                    cairo_set_line_width (cr, 2);
+                    CAIRO_SET_SOURCE_ARGB (cr, 0x80A0A0A0);
+                    cairo_rectangle (cr, sbx + 1, sby + 1, sbw - 2, sbh - 2);
+                    cairo_stroke (cr);
+                    if (s->x_scroll)
+                        cairoDrawRect (cr, 0x80000000,
+                                sbx + 2, sby + 2,
+                                knob.x - sbx - 2, sbh - 4);
+                    cairoDrawRect (cr, 0x80808080,
+                            knob.x, knob.y + 2,
+                            knob.w, knob.h - 4);
+                    if (sbx + sbw - knob.x - knob.w - 2 > 0)
+                        cairoDrawRect (cr, 0x80000000,
+                                knob.x + knob.w, sby + 2,
+                                sbx + sbw - knob.x - knob.w - 2, sbh - 4);
+                    cairo_restore (cr);
+                }
+            }
+        }
         cairo_restore (cr);
         matrix = m;
         clip = clip_save;
@@ -587,7 +682,7 @@ static Mrl *findActiveMrl (Node *n, bool *rp_or_smil) {
                  ActorAgent::AudioVideo == mrl->media_info->type))
             return mrl;
     }
-    for (Node *c = n->firstChild ().ptr (); c; c = c->nextSibling ())
+    for (Node *c = n->firstChild (); c; c = c->nextSibling ())
         if (c->active ()) {
             Mrl *m = findActiveMrl (c, rp_or_smil);
             if (m)
@@ -646,21 +741,25 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::ImageMediaType * img) {
     }
     if (!img->media_info->media)
         return;
-    ImageActor *im = static_cast <ImageActor *> (img->media_info->media);
-    ImageData *id = im ? im->cached_img.ptr () : NULL;
-    if (!id || im->isEmpty () || img->width <= 0 || img->height <= 0) {
-        s->remove();
-        return;
-    }
     SRect rect = s->bounds;
     Single x = rect.x ();
     Single y = rect.y ();
     Single w = rect.width();
     Single h = rect.height();
     matrix.getXYWH (x, y, w, h);
+
     IRect clip_rect = clip.intersect (IRect (x, y, w, h));
     if (clip_rect.isEmpty ())
         return;
+
+    ImageActor *im = static_cast <ImageActor *> (img->media_info->media);
+    ImageData *id = im ? im->cached_img.ptr () : NULL;
+    if (id && id->flags == ImageData::ImageScalable)
+        penalty += im->render (cairo_surface, w, h);
+    if (!id || im->isEmpty () || img->width <= 0 || img->height <= 0) {
+        s->remove();
+        return;
+    }
     if (!s->surface || s->dirty)
         id->copyImage (s, w, h, cairo_surface, &penalty, img->pan_zoom);
     if (id->has_alpha)
@@ -671,6 +770,29 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::ImageMediaType * img) {
     s->dirty = false;
 }
 
+static void calculateTextDimensions (PangoFontDescription *desc,
+        const char *text, Single w, Single h,
+        int *pxw, int *pxh, bool markup_text) {
+    cairo_surface_t *img_surf = cairo_image_surface_create (
+            CAIRO_FORMAT_RGB24, (int) w, h);
+    cairo_t *cr_txt = cairo_create (img_surf);
+    PangoLayout *layout = pango_cairo_create_layout (cr_txt);
+    pango_layout_set_width (layout, 1.0 * w * PANGO_SCALE);
+    pango_layout_set_wrap (layout, PANGO_WRAP_WORD_CHAR);
+    if (markup_text)
+        pango_layout_set_markup (layout, text, -1);
+    else
+        pango_layout_set_text (layout, text, -1);
+    pango_layout_set_font_description (layout, desc);
+
+    pango_cairo_show_layout (cr_txt, layout);
+    pango_layout_get_pixel_size (layout, pxw, pxh);
+
+    g_object_unref (layout);
+    cairo_destroy (cr_txt);
+    cairo_surface_destroy (img_surf);
+}
+
 KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::TextMediaType * txt) {
     if (!txt->media_info || !txt->media_info->media)
         return;
@@ -704,9 +826,19 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::TextMediaType * txt) {
         QPixmap pix = QPixmap::grabWidget (edit, rect.x () - (int) xoff,
                 rect.y () - (int) yoff, rect.width (), rect.height ());*/
 
-        cairo_surface_t *draw = cairo_surface_create_similar (cairo_surface,
-                CAIRO_CONTENT_COLOR, (int) w, (int) h * 2); //FIXME
-        cairo_t *cr_txt = cairo_create (draw);
+        int pxw, pxh;
+        Single ft_size = w * txt->font_size / rect.width ();
+        const char *text = (const char *) tm->text;
+
+        PangoFontDescription *desc = pango_font_description_new ();
+        pango_font_description_set_family (desc, "Sans");
+        pango_font_description_set_absolute_size (desc, PANGO_SCALE * ft_size);
+
+        calculateTextDimensions (desc, text, w, 2 * ft_size, &pxw, &pxh, false);
+        s->surface = cairo_surface_create_similar (cairo_surface,
+                CAIRO_CONTENT_COLOR, pxw, pxh);
+        cairo_t *cr_txt = cairo_create (s->surface);
+
         CAIRO_SET_SOURCE_RGB (cr_txt, txt->background_color);
         cairo_paint (cr_txt);
 
@@ -714,34 +846,15 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::TextMediaType * txt) {
         PangoLayout *layout = pango_cairo_create_layout (cr_txt);
         pango_layout_set_width (layout, 1.0 * w * PANGO_SCALE);
         pango_layout_set_wrap (layout, PANGO_WRAP_WORD_CHAR);
-        pango_layout_set_text (layout, (const char *)tm->text, -1);
-
-        PangoFontDescription *desc = pango_font_description_new ();
-        pango_font_description_set_family (desc, "Sans");
-        pango_font_description_set_absolute_size (desc,
-                PANGO_SCALE * (w * txt->font_size / rect.width ()));
+        pango_layout_set_text (layout, text, -1);
         pango_layout_set_font_description (layout, desc);
-        pango_font_description_free (desc);
 
         pango_cairo_show_layout (cr_txt, layout);
-        int pxw, pxh;
-        pango_layout_get_pixel_size (layout, &pxw, &pxh);
+
+        pango_font_description_free (desc);
         g_object_unref (layout);
         cairo_destroy (cr_txt);
 
-        cairo_pattern_t *pat = cairo_pattern_create_for_surface (draw);
-        cairo_pattern_set_extend (pat, CAIRO_EXTEND_NONE);
-        s->surface = cairo_surface_create_similar (cairo_surface,
-                CAIRO_CONTENT_COLOR, pxw, pxh);
-        cr_txt = cairo_create (s->surface);
-        cairo_set_operator (cr_txt, CAIRO_OPERATOR_SOURCE);
-        cairo_set_source (cr_txt, pat);
-        cairo_paint (cr_txt);
-
-        cairo_destroy (cr_txt);
-        cairo_pattern_destroy (pat);
-        cairo_surface_destroy (draw);
-
         // update bounds rect
         Single sx = x, sy = y, sw = pxw, sh = pxh;
         matrix.invXYWH (sx, sy, sw, sh);
@@ -766,18 +879,21 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Brush * brush) {
     //debugLog() << "Visit " << brush->nodeName() << endl;
     Surface *s = (Surface *) brush->message (MsgQueryRoleDisplay);
     if (s) {
-        cairo_save (cr);
         opacity = 1.0;
         SRect rect = s->bounds;
-        Single x, y, w = rect.width(), h = rect.height();
+        Single x = rect.x(), y = rect.y(), w = rect.width(), h = rect.height();
         matrix.getXYWH (x, y, w, h);
+        IRect clip_rect = clip.intersect (IRect (x, y, w, h));
+        if (clip_rect.isEmpty ())
+            return;
+        cairo_save (cr);
         unsigned int color = Color (brush->param ("color")).rgb ();
         if (brush->active_trans) {
             cur_media = brush;
             cur_pat = NULL;
             brush->active_trans->accept (this);
         } else {
-            cairo_rectangle (cr, (int) x, (int) y, (int) w, (int) h);
+            cairo_rectangle (cr, clip_rect.x, clip_rect.y, clip_rect.w, clip_rect.h);
         }
         opacity *= brush->opacity / 100.0;
         if (opacity < 0.99) {
@@ -809,27 +925,45 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::SmilText *txt) {
 
     if (!s->surface) {
 
+        Single ft_size = w * 14 / rect.width ();
+        int pxw, pxh;
+        String rtext = txt->richText ();
+        const char *text = (const char *) rtext;
+
+        PangoFontDescription *desc = pango_font_description_new ();
+        pango_font_description_set_family (desc, "Sans");
+        pango_font_description_set_absolute_size (desc, PANGO_SCALE * ft_size);
+
+        calculateTextDimensions (desc, text, w, 2 * ft_size, &pxw, &pxh, true);
+
         s->surface = cairo_surface_create_similar (cairo_surface,
-                CAIRO_CONTENT_COLOR_ALPHA, (int) w, (int) h);
+                CAIRO_CONTENT_COLOR_ALPHA, (int) w, pxh);
         cairo_t *cr_txt = cairo_create (s->surface);
 
         CAIRO_SET_SOURCE_RGB (cr_txt, 0);
         PangoLayout *layout = pango_cairo_create_layout (cr_txt);
         pango_layout_set_width (layout, 1.0 * w * PANGO_SCALE);
         pango_layout_set_wrap (layout, PANGO_WRAP_WORD_CHAR);
-
-        pango_layout_set_markup (layout, (const char *) txt->richText (), -1);
-
-        PangoFontDescription *desc = pango_font_description_new ();
-        pango_font_description_set_family (desc, "Sans");
-        pango_font_description_set_absolute_size (desc,
-                PANGO_SCALE * (w * 14 / rect.width ()));
+        pango_layout_set_markup (layout, text, -1);
         pango_layout_set_font_description (layout, desc);
-        pango_font_description_free (desc);
 
         pango_cairo_show_layout (cr_txt, layout);
+
+        pango_font_description_free (desc);
         g_object_unref (layout);
         cairo_destroy (cr_txt);
+
+        // update bounds rect
+        Single sx = x, sy = y, sw = w, sh = pxh;
+        matrix.invXYWH (sx, sy, sw, sh);
+        s->bounds = SRect (sx, sy, sw, sh);
+
+        // update coord. for painting below
+        x = s->bounds.x ();
+        y = s->bounds.y ();
+        w = s->bounds.width ();
+        h = s->bounds.height ();
+        matrix.getXYWH (x, y, w, h);
     }
     IRect clip_rect = clip.intersect (IRect (x, y, w, h));
     if (!clip_rect.isEmpty ()) {
@@ -1093,6 +1227,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::ViewChange * vc) {
 namespace KMPlayer {
 
 class KMPLAYER_NO_EXPORT MouseVisitor : public Visitor {
+    ViewArea *view_area;
     Matrix matrix;
     NodePtr node;
     MessageType event;
@@ -1100,7 +1235,7 @@ class KMPLAYER_NO_EXPORT MouseVisitor : public Visitor {
     bool handled;
     bool bubble_up;
 public:
-    MouseVisitor (MessageType evt, int x, int y);
+    MouseVisitor (ViewArea *v, MessageType evt, int x, int y);
     KDE_NO_CDTOR_EXPORT ~MouseVisitor () {}
     using Visitor::visit;
     void visit (Node * n);
@@ -1117,8 +1252,9 @@ public:
 } // namespace
 
 KDE_NO_CDTOR_EXPORT
-MouseVisitor::MouseVisitor (MessageType evt, int a, int b)
-    : event (evt), x (a), y (b), handled (false), bubble_up (false) {
+MouseVisitor::MouseVisitor (ViewArea *v, MessageType evt, int a, int b)
+  : view_area (v), event (evt), x (a), y (b),
+    handled (false), bubble_up (false) {
 }
 
 KDE_NO_EXPORT void MouseVisitor::visit (Node * n) {
@@ -1153,17 +1289,50 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::Layout * layout) {
 }
 
 KDE_NO_EXPORT void MouseVisitor::visit (SMIL::RegionBase *region) {
-    if (region->message (MsgQueryRoleDisplay)) {
-        SRect rect = region->region_surface->bounds;
+    Surface *s = (Surface *) region->message (MsgQueryRoleDisplay);
+    if (s) {
+        SRect rect = s->bounds;
         Single rx = rect.x(), ry = rect.y(), rw = rect.width(), rh = rect.height();
         matrix.getXYWH (rx, ry, rw, rh);
         handled = false;
         bool inside = x > rx && x < rx+rw && y > ry && y< ry+rh;
         if (!inside && (event == MsgEventClicked || !region->has_mouse))
             return;
+
+        if (event == MsgEventClicked && !s->virtual_size.isEmpty () &&
+                x > rx + rw - REGION_SCROLLBAR_WIDTH) {
+            const int sbh = rh - REGION_SCROLLBAR_WIDTH;
+            const int vy = s->virtual_size.height;
+            const int knob_h = sbh * (int) rh / vy;
+            int knob_y = y - (int) ry - 0.5 * knob_h;
+            if (knob_y < 0)
+                knob_y = 0;
+            else if (knob_y + knob_h > sbh)
+                knob_y = sbh - knob_h;
+            s->y_scroll = vy * knob_y / sbh;
+            view_area->scheduleRepaint (IRect (rx, ry, rw, rh));
+            return;
+        }
+        if (event == MsgEventClicked && !s->virtual_size.isEmpty () &&
+                y > ry + rh - REGION_SCROLLBAR_WIDTH) {
+            const int sbw = rw - REGION_SCROLLBAR_WIDTH;
+            const int vw = s->virtual_size.width;
+            const int knob_w = sbw * (int) rw / vw;
+            int knob_x = x - (int) rx - 0.5 * knob_w;
+            if (knob_x < 0)
+                knob_x = 0;
+            else if (knob_x + knob_w > sbw)
+                knob_x = sbw - knob_w;
+            s->x_scroll = vw * knob_x / sbw;
+            view_area->scheduleRepaint (IRect (rx, ry, rw, rh));
+            return;
+        }
+
         Matrix m = matrix;
         matrix = Matrix (rect.x(), rect.y(), 1.0, 1.0);
         matrix.transform (m);
+        if (!s->virtual_size.isEmpty ())
+            matrix.translate (-s->x_scroll, -s->y_scroll);
         bubble_up = false;
 
         bool child_handled = false;
@@ -1466,7 +1635,9 @@ KDE_NO_EXPORT void ViewArea::reset () {
 
 KDE_NO_EXPORT void ViewArea::mousePressEvent (int x, int y) {
     if (surface->node) {
-        MouseVisitor visitor (MsgEventClicked, x, y);
+        struct timeval tv;
+        surface->node->document ()->timeOfDay (tv); // update last_event_time
+        MouseVisitor visitor (this, MsgEventClicked, x, y);
         surface->node->accept (&visitor);
     }
 }
